/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __READ_H
#define __READ_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <string>
#include <set>
#include <map>
#include <istream>

/**
	@author Michael Moritz
	@brief Reader reads a number of lines from an input stream
	until it finds and empty line.
	
	@brief Reader::AttributeSet is used for passing the keys Reader
	should look out for. Any other keys will be ignored.
**/
class Reader
{
private:
	typedef std::map<std::string,std::string>AttributeMap;
public:
	typedef std::set<std::string>AttributeSet;
	/** 
		@returns the number of attributes from AttributeSet parsed or
				 -1 if EOF
	**/
	int parse(const AttributeSet&, std::istream &);
	
	/** reset the stored data */
	void reset() { _map.clear(); _input = ""; }
	
	/**
		@returns a @c string that contains the complete input, 
		including anything that has	been ignored, seperated by
		" " if in verbose mode 
	**/
	const std::string& getInput() const { return _input; }
	
	/**
		@returns a @c string if the key has been found,
		otherwise the @e not-found-string which can be checked
		for with isNotFound 
	**/
	const std::string& getValue(const std::string& key) const {
		AttributeMap::const_iterator itr = _map.find(key);
		if(itr != _map.end())
			return itr->second;
		else
			return _s_notfound;
	}
	
	//!deprecated
	bool isNotFound(const std::string& value) const {
		return( value == _s_notfound );
	}
private:
	AttributeMap _map;
	static const std::string _s_notfound;
	std::string _input;
};

#endif

