/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef STRTRIM_H
#define STRTRIM_H

#include <string>
#include <iostream>

/**
@brief StrTrim a simple but handy 5 line string trimmer
@author Michael Moritz

StrTrim has the advantage of extensively using std::string functions.
It should be quite slow
*/
class StrTrim{
public:
    StrTrim(const std::string& tobedel=" ") : _tobedel(tobedel) { };
	std::string& trimFront(std::string& input) const 
	{
		while( !input.empty() && (_tobedel.find(input[0]) != std::string::npos) )
			input.erase(0,1);
		return input;
	}
	std::string& trimBack(std::string& input) const 
	{
		std::string::iterator itr = input.end();
		while( !input.empty() && (_tobedel.find(*(--itr)) != std::string::npos) )
			input.erase(itr);
		return input;
	}
	std::string& trim(std::string& input) const 
	{
		trimFront(input);
		return trimBack(input);
	}
    ~StrTrim() {};
private:
	std::string _tobedel;
};

#endif
